==> musikloud_1.4.0.tar.gz: musikloud-1.4.0/musikloudapp/src/base/thumbnailcache.cpp <==
#include "thumbnailcache.h"
#include "thumbnaildownloader.h"
#include <QThread>
#include <QCache>
#include <QImage>
#include <QUrl>
#include <QNetworkReply>
#include <QCoreApplication>
#include <qcoreevent.h>

#if (QT_VERSION < QT_VERSION_CHECK(4, 7, 0))
uint qHash(const QUrl &url) { return qHash(url.toString()); }
#endif

class Thumbnail
{

public:
    Thumbnail(const QUrl &url)
        : url(url)
    {
    }

    const QUrl url;
    QImage image;
};

class ThumbnailEvent : public QEvent
{

public:
    ThumbnailEvent(const QUrl &url, const QImage &image)
        : QEvent(QEvent::User)
        , url(url)
        , image(image)
    {
    }

    const QUrl url;
    const QImage image;
};

ThumbnailCache* cacheInstance = 0;
QThread* cacheThread = 0;
QCache<QUrl, Thumbnail> thumbnails;

ThumbnailCache::ThumbnailCache(QObject *parent) :
    QObject(parent)
{
    if (!cacheInstance) {
        cacheInstance = this;
        cacheThread = new QThread(this);
        cacheThread->start();
        this->connect(cacheInstance, SIGNAL(destroyed()), cacheThread, SLOT(quit()));
    }

    this->moveToThread(cacheThread);

    ThumbnailDownloader *downloader = new ThumbnailDownloader;
    this->connect(this, SIGNAL(downloadThumbnail(QUrl)), downloader, SLOT(downloadThumbnail(QUrl)));
    this->connect(downloader, SIGNAL(gotThumbnail(QNetworkReply*)), this, SLOT(onThumbnailDownloaded(QNetworkReply*)));
    this->connect(this, SIGNAL(destroyed()), downloader, SLOT(deleteLater()));
}

ThumbnailCache::~ThumbnailCache() {}

ThumbnailCache* ThumbnailCache::instance() {
    return cacheInstance;
}

QImage ThumbnailCache::thumbnail(const QUrl &url, const QSize &scale) {
    if (Thumbnail *thumbnail = thumbnails.object(url)) {
        return (!scale.isEmpty()) && (!thumbnail->image.isNull())
                ? thumbnail->image.scaled(scale, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)
                : thumbnail->image;
    }
    else {
        m_urls.enqueue(url);
        thumbnails.insert(url, new Thumbnail(url));

        if (m_urls.size() == 1) {
            emit downloadThumbnail(url);
        }

        return QImage();
    }
}

void ThumbnailCache::nextThumbnail() {
    if (!m_urls.isEmpty()) {
        emit downloadThumbnail(m_urls.dequeue());
    }
}

bool ThumbnailCache::event(QEvent *event) {
    if (event->type() == QEvent::User) {
        ThumbnailEvent *thumbnailEvent = static_cast<ThumbnailEvent *>(event);

        if (Thumbnail *thumbnail = thumbnails.object(thumbnailEvent->url)) {
            thumbnail->image = thumbnailEvent->image;

            emit thumbnailReady();

            nextThumbnail();
        }

        return true;
    }
    else {
        return QObject::event(event);
    }
}

void ThumbnailCache::onThumbnailDownloaded(QNetworkReply *reply) {
    QUrl url = reply->request().url();
    QImage image;
    image.loadFromData(reply->readAll());
    reply->deleteLater();
    QCoreApplication::postEvent(this, new ThumbnailEvent(url, image));
}

==> musikloud_1.4.4.tar.gz: musikloud-1.4.4/musikloudapp/src/base/thumbnailcache.cpp <==
#include "thumbnailcache.h"
#include "thumbnaildownloader.h"
#include <QThread>
#include <QCache>
#include <QImage>
#include <QUrl>
#include <QNetworkReply>
#include <QCoreApplication>
#include <qcoreevent.h>

#if (QT_VERSION < QT_VERSION_CHECK(4, 7, 0))
uint qHash(const QUrl &url) { return qHash(url.toString()); }
#endif

class Thumbnail
{

public:
    Thumbnail(const QUrl &url)
        : url(url)
    {
    }

    const QUrl url;
    QImage image;
};

class ThumbnailEvent : public QEvent
{

public:
    ThumbnailEvent(const QUrl &url, const QImage &image)
        : QEvent(QEvent::User)
        , url(url)
        , image(image)
    {
    }

    const QUrl url;
    const QImage image;
};

ThumbnailCache* ThumbnailCache::m_instance = 0;
QThread* cacheThread = 0;
QCache<QUrl, Thumbnail> thumbnails;

ThumbnailCache::ThumbnailCache(QObject *parent) :
    QObject(parent)
{
    if (!m_instance) {
        m_instance = this;
        cacheThread = new QThread(this);
        cacheThread->start();
        this->connect(m_instance, SIGNAL(destroyed()), cacheThread, SLOT(quit()));
    }

    this->moveToThread(cacheThread);

    ThumbnailDownloader *downloader = new ThumbnailDownloader;
    this->connect(this, SIGNAL(downloadThumbnail(QUrl)), downloader, SLOT(downloadThumbnail(QUrl)));
    this->connect(downloader, SIGNAL(gotThumbnail(QNetworkReply*)), this, SLOT(onThumbnailDownloaded(QNetworkReply*)));
    this->connect(this, SIGNAL(destroyed()), downloader, SLOT(deleteLater()));
}

ThumbnailCache::~ThumbnailCache() {}

ThumbnailCache* ThumbnailCache::instance() {
    return !m_instance ? new ThumbnailCache : m_instance;
}

QImage ThumbnailCache::thumbnail(const QUrl &url, const QSize &scale) {
    if (Thumbnail *thumbnail = thumbnails.object(url)) {
        return (!scale.isEmpty()) && (!thumbnail->image.isNull())
                ? thumbnail->image.scaled(scale, Qt::IgnoreAspectRatio, Qt::SmoothTransformation)
                : thumbnail->image;
    }
    else {
        m_urls.enqueue(url);
        thumbnails.insert(url, new Thumbnail(url));

        if (m_urls.size() == 1) {
            emit downloadThumbnail(url);
        }

        return QImage();
    }
}

void ThumbnailCache::nextThumbnail() {
    if (!m_urls.isEmpty()) {
        emit downloadThumbnail(m_urls.dequeue());
    }
}

bool ThumbnailCache::event(QEvent *event) {
    if (event->type() == QEvent::User) {
        ThumbnailEvent *thumbnailEvent = static_cast<ThumbnailEvent *>(event);

        if (Thumbnail *thumbnail = thumbnails.object(thumbnailEvent->url)) {
            thumbnail->image = thumbnailEvent->image;

            emit thumbnailReady();

            nextThumbnail();
        }

        return true;
    }
    else {
        return QObject::event(event);
    }
}

void ThumbnailCache::onThumbnailDownloaded(QNetworkReply *reply) {
    QUrl url = reply->request().url();
    QImage image;
    image.loadFromData(reply->readAll());
    reply->deleteLater();
    QCoreApplication::postEvent(this, new ThumbnailEvent(url, image));
}
